@@ -27,6 +27,45 @@ import javabridge
 
 logger = logging.getLogger(__name__)
 
+
+class JavaError(ValueError):
+    '''An error caused by using the Javabridge incorrectly'''
+    def __init__(self, message=None):
+        super(JavaError,self).__init__(message)
+
+
+class JVMNotFoundError(JavaError):
+    '''Failed to find The Java Runtime Environment'''
+    def __init__(self):
+        super(JVMNotFoundError, self).__init__("Can't find the Java Virtual Machine")
+        
+
+class JavaException(Exception):
+    '''Represents a Java exception thrown inside the JVM'''
+    def __init__(self, throwable):
+        '''Initialize by calling exception_occurred'''
+        env = get_env()
+        env.exception_describe()
+        self.throwable = throwable
+        try:
+            if self.throwable is None:
+                raise ValueError("Tried to create a JavaException but there was no current exception")
+            #
+            # The following has to be done by hand because the exception can't be
+            # cleared at this point
+            #
+            klass = env.get_object_class(self.throwable)
+            method_id = env.get_method_id(klass, 'getMessage', 
+                                          '()Ljava/lang/String;')
+            if method_id is not None:
+                message = env.call_method(self.throwable, method_id)
+                if message is not None:
+                    message = env.get_string_utf(message)
+                    super(JavaException, self).__init__(message)
+        finally:
+            env.exception_clear()
+
+
 def _find_jvm_windows():
     # Look for JAVA_HOME and in the registry
     java_home = find_javahome()
@@ -54,71 +93,21 @@ def _find_jvm_mac():
     os.environ['PATH'] = os.environ['PATH'] + ':' + jvm_dir
     return jvm_dir
 
-def _find_jvm_linux():
-    #
-    # Run the findlibjvm program which uses ``java.library.path`` to
-    # find the search path for the JVM.
-    #
-    import ctypes
-    class_path = os.path.join(os.path.dirname(__file__), 'jars', 'findlibjvm.jar')
-    p = subprocess.Popen(["java","-cp", class_path, "findlibjvm"],
-                         stdout=subprocess.PIPE)
-    stdout, stderr = p.communicate()
-    jvm_dir = stdout.strip()
-    ctypes.CDLL(os.path.join(jvm_dir, "libjvm.so"))
-    return jvm_dir
-
 def _find_jvm():
     jvm_dir = None
     if sys.platform.startswith('win'):
         jvm_dir = _find_jvm_windows()
+        if jvm_dir is None:
+            raise JavaNotFoundException()
     elif sys.platform == 'darwin':
         jvm_dir = _find_jvm_mac()
-    elif sys.platform.startswith('linux'):
-        jvm_dir = _find_jvm_linux()
-
-    if jvm_dir is None:
-        raise JVMNotFoundError()
     return jvm_dir
 
-class JavaError(ValueError):
-    '''An error caused by using the Javabridge incorrectly'''
-    def __init__(self, message=None):
-        super(JavaError,self).__init__(message)
-
-class JVMNotFoundError(JavaError):
-    '''Failed to find The Java Runtime Environment'''
-    def __init__(self):
-        super(JVMNotFoundError, self).__init__("Can't find the Java Virtual Machine")
-        
-class JavaException(Exception):
-    '''Represents a Java exception thrown inside the JVM'''
-    def __init__(self, throwable):
-        '''Initialize by calling exception_occurred'''
-        env = get_env()
-        env.exception_describe()
-        self.throwable = throwable
-        try:
-            if self.throwable is None:
-                raise ValueError("Tried to create a JavaException but there was no current exception")
-            #
-            # The following has to be done by hand because the exception can't be
-            # cleared at this point
-            #
-            klass = env.get_object_class(self.throwable)
-            method_id = env.get_method_id(klass, 'getMessage', 
-                                          '()Ljava/lang/String;')
-            if method_id is not None:
-                message = env.call_method(self.throwable, method_id)
-                if message is not None:
-                    message = env.get_string_utf(message)
-                    super(JavaException, self).__init__(message)
-        finally:
-            env.exception_clear()
 
 if sys.platform == "win32":
     # Need to fix up executable path to pick up jvm.dll
     os.environ["PATH"] = os.environ["PATH"] + os.pathsep + _find_jvm()
+
 import _javabridge
 __vm = None
 __wake_event = threading.Event()
@@ -131,6 +120,7 @@ __run_headless = False
 
 RQCLS = "org/cellprofiler/runnablequeue/RunnableQueue"
 
+
 class AtExit(object):
     '''AtExit runs a function as the main thread exits from the __main__ function
     
@@ -159,7 +149,10 @@ def start_vm(args=[], class_path=None, max_heap_size=None, run_headless=False):
       for the VM. In particular, strings on the form
       ``"-D<name>=<value>"`` are used to set Java system
       properties. For other startup options, see `"The Invocation API"
-      <http://docs.oracle.com/javase/6/docs/technotes/guides/jni/spec/invocation.html>`_.
+      <http://docs.oracle.com/javase/6/docs/technotes/guides/jni/spec/invocation.html>`_. Options
+      that set the class path (``-cp``, ``-classpath``, and
+      ``-Djava.class.path``) are not allowed here; instead, use the
+      `class_path` keyword argument.
 
     :param class_path: a list of strings constituting a class search
       path. Each string can be a directory, JAR archive, or ZIP
@@ -181,6 +174,11 @@ def start_vm(args=[], class_path=None, max_heap_size=None, run_headless=False):
     global __vm
     global __start_thread
 
+    # Put this before the __vm check so the unit test can test it even
+    # though the JVM is already started.
+    if '-cp' in args or '-classpath' in args or any(arg.startswith('-Djava.class.path=') for arg in args):
+        raise ValueError("Cannot set Java class path in the \"args\" argument to start_vm. Use the class_path keyword argument to javabridge.start_vm instead.")
+
     _find_jvm()
     
     if __vm is not None:
@@ -285,9 +283,11 @@ def start_vm(args=[], class_path=None, max_heap_size=None, run_headless=False):
 def unwrap_javascript(o):
     '''Unwrap an object such as NativeJavaObject
     
-    o - an object, possibly implementing org.mozilla.javascript.Wrapper
+    :param o: an object, possibly implementing org.mozilla.javascript.Wrapper
     
-    return nice version
+    :returns: result of calling the wrapper's unwrap method if a wrapper,
+              otherwise the unboxed value for boxed types such as
+              java.lang.Integer, and if not boxed, return the Java object.
     '''
     if is_instance_of(o, "org/mozilla/javascript/Wrapper"):
         o = call(o, "unwrap", "()Ljava/lang/Object;")
@@ -754,6 +754,8 @@ def get_env():
     Note: call start_vm() and attach() before calling this
     '''
     global __thread_local_env
+    if not hasattr(__thread_local_env, 'env'):
+        attach()
     return __thread_local_env.env
 
 def detach():
@@ -1622,8 +1624,13 @@ def iterate_java(iterator, fn_wrapper=None):
     '''
     global iterator_has_next_id, iterator_next_id
     env = get_env()
+    iterator_class = env.find_class("java/util/Iterator")
+    if not isinstance(iterator, javabridge.JB_Object):
+        raise JavaError("%s is not a Javabridge JB_Object" % repr(iterator))
+    if not env.is_instance_of(iterator, iterator_class):
+        raise JavaError("%s does not implement the java.util.Iterator interface" %
+                        get_class_wrapper(iterator).getCanonicalName())
     if iterator_has_next_id is None:
-        iterator_class = env.find_class("java/util/Iterator")
         iterator_has_next_id = env.get_method_id(iterator_class, "hasNext", "()Z")
         iterator_next_id = env.get_method_id(iterator_class, "next", "()Ljava/lang/Object;")
     while(True):
